@@ -12,43 +12,35 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
-import gzip
 import hashlib
 import os
-import requests
-import six
 import time
 
-from oslo_concurrency import processutils
-from oslo_log import log
-
 from ironic_lib import disk_utils
+from oslo_concurrency import processutils
+from oslo_config import cfg
+from oslo_log import log
+import requests
+import six
+
 from ironic_python_agent import errors
 from ironic_python_agent.extensions import base
 from ironic_python_agent import hardware
 from ironic_python_agent import utils
 
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 IMAGE_CHUNK_SIZE = 1024 * 1024  # 1MB
 
 
-def _configdrive_location():
-    """Get the configdrive location in the local file system.
-
-    :returns: The full, absolute path to the configdrive as a string.
-    """
-    return '/tmp/configdrive'
-
-
 def _image_location(image_info):
     """Get the location of the image in the local file system.
 
     :param image_info: Image information dictionary.
     :returns: The full, absolute path to the image as a string.
     """
-    return '/tmp/{0}'.format(image_info['id'])
+    return '/tmp/{}'.format(image_info['id'])
 
 
 def _path_to_script(script):
@@ -83,8 +75,8 @@ def _write_partition_image(image, image_info, device):
     root_mb = image_info['root_mb']
     if image_mb > int(root_mb):
         msg = ('Root partition is too small for requested image. Image '
-               'virtual size: {0} MB, Root size: {1} MB').format(image_mb,
-                                                                 root_mb)
+               'virtual size: {} MB, Root size: {} MB').format(image_mb,
+                                                               root_mb)
         raise errors.InvalidCommandParamsError(msg)
     try:
         return disk_utils.work_on_disk(device, root_mb,
@@ -115,7 +107,7 @@ def _write_whole_disk_image(image, image_info, device):
     """
     script = _path_to_script('shell/write_image.sh')
     command = ['/bin/bash', script, image, device]
-    LOG.info('Writing image with command: {0}'.format(' '.join(command)))
+    LOG.info('Writing image with command: {}'.format(' '.join(command)))
     try:
         stdout, stderr = utils.execute(*command, check_exit_code=[0])
     except processutils.ProcessExecutionError as e:
@@ -139,93 +131,11 @@ def _write_image(image_info, device):
     else:
         _write_whole_disk_image(image, image_info, device)
     totaltime = time.time() - starttime
-    LOG.info('Image {0} written to device {1} in {2} seconds'.format(
+    LOG.info('Image {} written to device {} in {} seconds'.format(
              image, device, totaltime))
     return uuids
 
 
-def _configdrive_is_url(configdrive):
-    """Determine if the configdrive location looks like an HTTP(S) URL.
-
-    :param configdrive: Location of the configdrive as a string.
-    :returns: True if configdrive looks like an HTTP(S) URL, False otherwise.
-    """
-    return (configdrive.startswith('http://')
-            or configdrive.startswith('https://'))
-
-
-def _download_configdrive_to_file(configdrive, filename):
-    """Download the configdrive to a local file.
-
-    :param configdrive: The URL of the configdrive.
-    :param filename: The filename of where to store the configdrive locally.
-    """
-    content = requests.get(configdrive).content
-    _write_configdrive_to_file(content, filename)
-
-
-def _write_configdrive_to_file(configdrive, filename):
-    """Writes the configdrive to a file.
-
-    Note that the contents of the configdrive are expected to be gzipped and
-    base64 encoded.
-
-    :param configdrive: Contents of the configdrive file.
-    :param filename: The filename of where to write the configdrive.
-    """
-    LOG.debug('Writing configdrive to {0}'.format(filename))
-    # configdrive data is base64'd, decode it first
-    data = six.StringIO(base64.b64decode(configdrive))
-    gunzipped = gzip.GzipFile('configdrive', 'rb', 9, data)
-    with open(filename, 'wb') as f:
-        f.write(gunzipped.read())
-    gunzipped.close()
-
-
-def _write_configdrive_to_partition(configdrive, device):
-    """Writes the configdrive to a partition on the given device.
-
-    :param configdrive: A string containing the location of the config drive
-                        as a URL OR the contents of the configdrive which
-                        must be gzipped and base64 encoded.
-    :param device: The disk name, as a string, on which to store the image.
-                   Example: '/dev/sda'
-
-    :raises: ConfigDriveTooLargeError if the configdrive contents are too
-             large to store on the given device.
-    """
-    filename = _configdrive_location()
-    if _configdrive_is_url(configdrive):
-        _download_configdrive_to_file(configdrive, filename)
-    else:
-        _write_configdrive_to_file(configdrive, filename)
-
-    # check configdrive size before writing it
-    filesize = os.stat(filename).st_size
-    if filesize > (64 * 1024 * 1024):
-        raise errors.ConfigDriveTooLargeError(filename, filesize)
-
-    starttime = time.time()
-    script = _path_to_script('shell/copy_configdrive_to_disk.sh')
-    command = ['/bin/bash', script, filename, device]
-    LOG.info('copying configdrive to disk with command {0}'.format(
-             ' '.join(command)))
-
-    try:
-        stdout, stderr = utils.execute(*command, check_exit_code=[0])
-    except processutils.ProcessExecutionError as e:
-        raise errors.ConfigDriveWriteError(device,
-                                           e.exit_code,
-                                           e.stdout,
-                                           e.stderr)
-
-    totaltime = time.time() - starttime
-    LOG.info('configdrive copied from {0} to {1} in {2} seconds'.format(
-             filename,
-             device,
-             totaltime))
-
-
 def _message_format(msg, image_info, device, partition_uuids):
     """Helper method to get and populate different messages."""
     message = None
@@ -236,15 +146,25 @@ def _message_format(msg, image_info, device, partition_uuids):
             partition_uuids.get('efi system partition uuid'))
         if (image_info.get('deploy_boot_mode') == 'uefi' and
                 image_info.get('boot_option') == 'local'):
-            result_msg = msg + 'root_uuid={2} efi_system_partition_uuid={3}'
+            result_msg = msg + 'root_uuid={} efi_system_partition_uuid={}'
             message = result_msg.format(image_info['id'], device,
                                         root_uuid,
                                         efi_system_partition_uuid)
         else:
-            result_msg = msg + 'root_uuid={2}'
+            result_msg = msg + 'root_uuid={}'
             message = result_msg.format(image_info['id'], device, root_uuid)
     else:
-        message = result_msg.format(image_info['id'], device)
+        try:
+            # NOTE(TheJulia): ironic-lib disk_utils.get_disk_identifier
+            # can raise OSError if hexdump is not found.
+            root_uuid = disk_utils.get_disk_identifier(device)
+            result_msg = msg + 'root_uuid={}'
+            message = result_msg.format(image_info['id'], device, root_uuid)
+        except OSError as e:
+            LOG.warning('Failed to call get_disk_identifier: '
+                        'Unable to obtain the root_uuid parameter: '
+                        'The hexdump tool may be missing in IPA: %s', e)
+            message = result_msg.format(image_info['id'], device)
     return message
 
 
@@ -280,23 +200,21 @@ class ImageDownload(object):
         details = []
         for url in image_info['urls']:
             try:
-                LOG.info("Attempting to download image from {0}".format(url))
+                LOG.info("Attempting to download image from {}".format(url))
                 self._request = self._download_file(image_info, url)
             except errors.ImageDownloadError as e:
                 failtime = time.time() - self._time
-                log_msg = ('URL: {0}; time: {1} '
-                           'seconds. Error: {2}').format(
-                    url, failtime, e.details)
-                LOG.warning('Image download failed. %s', log_msg)
-                details += log_msg
+                log_msg = ('URL: {}; time: {} '
+                           'seconds. Error: {}').format(
+                    url, failtime, e.secondary_message)
+                LOG.warning(log_msg)
+                details.append(log_msg)
                 continue
             else:
                 break
         else:
-            details = '/n'.join(details)
-            msg = ('Image download failed for all URLs with following errors: '
-                   '{}'.format(details))
-            raise errors.ImageDownloadError(image_info['id'], msg)
+            details = '\n '.join(details)
+            raise errors.ImageDownloadError(image_info['id'], details)
 
     def _download_file(self, image_info, url):
         """Opens a download stream for the given URL.
@@ -311,10 +229,12 @@ class ImageDownload(object):
         if no_proxy:
             os.environ['no_proxy'] = no_proxy
         proxies = image_info.get('proxies', {})
-        resp = requests.get(url, stream=True, proxies=proxies)
+        verify, cert = utils.get_ssl_client_options(CONF)
+        resp = requests.get(url, stream=True, proxies=proxies,
+                            verify=verify, cert=cert)
         if resp.status_code != 200:
-            msg = ('Received status code {0} from {1}, expected 200. Response '
-                   'body: {2}').format(resp.status_code, url, resp.text)
+            msg = ('Received status code {} from {}, expected 200. Response '
+                   'body: {}').format(resp.status_code, url, resp.text)
             raise errors.ImageDownloadError(image_info['id'], msg)
         return resp
 
@@ -352,8 +272,8 @@ def _verify_image(image_info, image_location, checksum):
     :raises: ImageChecksumError if the checksum of the local image does not
              match the checksum as reported by glance in image_info.
     """
-    LOG.debug('Verifying image at {0} against MD5 checksum '
-              '{1}'.format(image_location, checksum))
+    LOG.debug('Verifying image at {} against MD5 checksum '
+              '{}'.format(image_location, checksum))
     if checksum != image_info['checksum']:
         LOG.error(errors.ImageChecksumError.details_str.format(
             image_location, image_info['id'],
@@ -379,13 +299,13 @@ def _download_image(image_info):
             for chunk in image_download:
                 f.write(chunk)
         except Exception as e:
-            msg = 'Unable to write image to {0}. Error: {1}'.format(
+            msg = 'Unable to write image to {}. Error: {}'.format(
                 image_location, str(e))
             raise errors.ImageDownloadError(image_info['id'], msg)
 
     totaltime = time.time() - starttime
-    LOG.info("Image downloaded from {0} in {1} seconds".format(image_location,
-                                                               totaltime))
+    LOG.info("Image downloaded from {} in {} seconds".format(image_location,
+                                                             totaltime))
     _verify_image(image_info, image_location, image_download.md5sum())
 
 
@@ -405,7 +325,7 @@ def _validate_image_info(ext, image_info=None, **kwargs):
 
     for field in ['id', 'urls', 'checksum']:
         if field not in image_info:
-            msg = 'Image is missing \'{0}\' field.'.format(field)
+            msg = 'Image is missing \'{}\' field.'.format(field)
             raise errors.InvalidCommandParamsError(msg)
 
     if type(image_info['urls']) != list or not image_info['urls']:
@@ -465,12 +385,12 @@ class StandbyExtension(base.BaseAgentExtension):
                 for chunk in image_download:
                     f.write(chunk)
             except Exception as e:
-                msg = 'Unable to write image to device {0}. Error: {1}'.format(
+                msg = 'Unable to write image to device {}. Error: {}'.format(
                       device, str(e))
                 raise errors.ImageDownloadError(image_info['id'], msg)
 
         totaltime = time.time() - starttime
-        LOG.info("Image streamed onto device {0} in {1} "
+        LOG.info("Image streamed onto device {} in {} "
                  "seconds".format(device, totaltime))
         # Verify if the checksum of the streamed image is correct
         _verify_image(image_info, device, image_download.md5sum())
@@ -492,13 +412,13 @@ class StandbyExtension(base.BaseAgentExtension):
         LOG.debug('Caching image %s', image_info['id'])
         device = hardware.dispatch_to_managers('get_os_install_device')
 
-        msg = 'image ({0}) already present on device {1} '
+        msg = 'image ({}) already present on device {} '
 
         if self.cached_image_id != image_info['id'] or force:
             LOG.debug('Already had %s cached, overwriting',
                       self.cached_image_id)
             self._cache_and_write_image(image_info, device)
-            msg = 'image ({0}) cached to device {1} '
+            msg = 'image ({}) cached to device {} '
 
         result_msg = _message_format(msg, image_info, device,
                                      self.partition_uuids)
@@ -527,7 +447,7 @@ class StandbyExtension(base.BaseAgentExtension):
         :raises: ImageChecksumError if the checksum of the local image does not
              match the checksum as reported by glance in image_info.
         :raises: ImageWriteError if writing the image fails.
-        :raises: ConfigDriveTooLargeError if the configdrive contents are too
+        :raises: InstanceDeployFailure if failed to create config drive.
              large to store on the given device.
         """
         LOG.debug('Preparing image %s', image_info['id'])
@@ -551,33 +471,47 @@ class StandbyExtension(base.BaseAgentExtension):
         # work_on_disk().
         if image_info.get('image_type') != 'partition':
             if configdrive is not None:
-                _write_configdrive_to_partition(configdrive, device)
-
-        msg = 'image ({0}) written to device {1} '
+                # Will use dummy value of 'local' for 'node_uuid',
+                # if it is not available. This is to handle scenario
+                # wherein new IPA is being used with older version
+                # of Ironic that did not pass 'node_uuid' in 'image_info'
+                node_uuid = image_info.get('node_uuid', 'local')
+                disk_utils.create_config_drive_partition(node_uuid,
+                                                         device,
+                                                         configdrive)
+        msg = 'image ({}) written to device {} '
         result_msg = _message_format(msg, image_info, device,
                                      self.partition_uuids)
         LOG.info(result_msg)
         return result_msg
 
-    def _run_shutdown_script(self, parameter):
-        """Runs the agent's shutdown script with the specified parameter.
+    def _run_shutdown_command(self, command):
+        """Run the shutdown or reboot command
 
-        The script only takes the following parameters:
-
-            -h : 'halts' the machine by turning the power off.
-            -r : 'runs' the installed image by rebooting the machine.
-
-        Only one parameter should be specified.
-
-        :param parameter: The parameter to send to the shutdown script.
-        :raises: SystemRebootError if calling the shutdown script returns an
-                 unsuccessful exit code.
+        :param command: A string having the command to be run.
+        :raises: InvalidCommandParamsError if the passed command is not
+            equal to poweroff or reboot.
+        :raises: SystemRebootError if the command errors out with an
+            unsuccessful exit code.
         """
-        script = _path_to_script('shell/shutdown.sh')
-        command = ['/bin/bash', script, parameter]
-        # this should never return if successful
+        if command not in ('reboot', 'poweroff'):
+            msg = (('Expected the command "poweroff" or "reboot" '
+                    'but received "%s".') % command)
+            raise errors.InvalidCommandParamsError(msg)
         try:
-            stdout, stderr = utils.execute(*command, check_exit_code=[0])
+            self.sync()
+        except errors.CommandExecutionError as e:
+            LOG.warning('Failed to sync file system buffers: % s', e)
+        try:
+            _, stderr = utils.execute(command, use_standard_locale=True,
+                                      check_exit_code=[0])
+            if 'ignoring request.' in stderr:
+                LOG.debug('%s command failed with error %s, '
+                          'falling back to sysrq-trigger.', command, stderr)
+                if command == 'poweroff':
+                    utils.execute("echo o > /proc/sysrq-trigger", shell=True)
+                elif command == 'reboot':
+                    utils.execute("echo b > /proc/sysrq-trigger", shell=True)
         except processutils.ProcessExecutionError as e:
             raise errors.SystemRebootError(e.exit_code, e.stdout, e.stderr)
 
@@ -585,13 +519,13 @@ class StandbyExtension(base.BaseAgentExtension):
     def run_image(self):
         """Runs image on agent's system via reboot."""
         LOG.info('Rebooting system')
-        self._run_shutdown_script('-r')
+        self._run_shutdown_command('reboot')
 
     @base.async_command('power_off')
     def power_off(self):
         """Powers off the agent's system."""
         LOG.info('Powering off system')
-        self._run_shutdown_script('-h')
+        self._run_shutdown_command('poweroff')
 
     @base.sync_command('sync')
     def sync(self):
